==> lang-impl-203.5981.155-sources.jar: com/intellij/util/indexing/diagnostic/dump/paths/indexedFilePaths.kt <==
// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.util.indexing.diagnostic.dump.paths

import com.intellij.openapi.application.runReadAction
import com.intellij.openapi.project.Project
import com.intellij.openapi.roots.impl.FilePropertyPusher
import com.intellij.openapi.util.io.FileTooBigException
import com.intellij.openapi.vfs.VirtualFile
import com.intellij.util.Base64
import com.intellij.util.indexing.FileBasedIndex
import com.intellij.util.indexing.FileContentImpl
import com.intellij.util.indexing.IndexedHashesSupport
import com.intellij.util.indexing.SubstitutedFileType

object IndexedFilePaths {

  const val TOO_LARGE_FILE = "<TOO LARGE>"

  const val FAILED_TO_LOAD = "<FAILED TO LOAD: %s>"

  fun createIndexedFilePath(fileOrDir: VirtualFile, project: Project): IndexedFilePath {
    val fileId = FileBasedIndex.getFileId(fileOrDir)
    val fileUrl = fileOrDir.url
    val fileType = if (fileOrDir.isDirectory) null else fileOrDir.fileType.name
    val substitutedFileType = if (fileOrDir.isDirectory) {
      null
    }
    else {
      runReadAction {
        SubstitutedFileType.substituteFileType(fileOrDir, fileOrDir.fileType, project).name.takeIf { it != fileType }
      }
    }
    val (contentHash, indexedHash) = try {
      val fileContent = FileContentImpl.createByFile(fileOrDir) as FileContentImpl
      runReadAction {
        val contentHash = IndexedHashesSupport.getBinaryContentHash(fileContent.content)
        val indexedHash = IndexedHashesSupport.calculateIndexedHash(fileContent, contentHash, false)
        Base64.encode(contentHash) to Base64.encode(indexedHash)
      }
    }
    catch (e: FileTooBigException) {
      TOO_LARGE_FILE to TOO_LARGE_FILE
    }
    catch (e: Throwable) {
      val msg = FAILED_TO_LOAD.format(e.message)
      msg to msg
    }

    val fileSize = if (fileOrDir.isDirectory) null else fileOrDir.length
    val portableFilePath = PortableFilePaths.getPortableFilePath(fileOrDir, project)
    val resolvedFile = PortableFilePaths.findFileByPath(portableFilePath, project)
    val allPusherValues = dumpFilePropertyPusherValues(fileOrDir, project).mapValues { it.value?.toString() ?: "<null-value>" }
    val indexedFilePath = IndexedFilePath(
      fileId,
      fileType,
      substitutedFileType,
      fileSize,
      fileUrl,
      portableFilePath,
      allPusherValues,
      indexedHash,
      contentHash
    )
    check(fileUrl == resolvedFile?.url) {
      buildString {
        appendln("File cannot be resolved")
        appendln("Original URL: $fileUrl")
        appendln("Resolved URL: ${resolvedFile?.url}")
        appendln(indexedFilePath.toString())
      }
    }
    return indexedFilePath
  }

  private fun dumpFilePropertyPusherValues(file: VirtualFile, project: Project): Map<String, Any?> {
    val map = hashMapOf<String, Any?>()
    FilePropertyPusher.EP_NAME.forEachExtensionSafe { pusher ->
      if (file.isDirectory && pusher.acceptsDirectory(file, project)
          || !file.isDirectory && pusher.acceptsFile(file, project)
      ) {
        map[pusher.pusherName] = pusher.getImmediateValue(project, file)
      }
    }
    return map
  }

  private val FilePropertyPusher<*>.pusherName: String
    get() = javaClass.name
      .removePrefix("com.")
      .removePrefix("intellij.")
      .removePrefix("jetbrains.")
      .replace("util.", "")
      .replace("impl.", "")
}

==> lang-impl-212.4746.96-sources.jar: com/intellij/util/indexing/diagnostic/dump/paths/indexedFilePaths.kt <==
// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.util.indexing.diagnostic.dump.paths

import com.intellij.openapi.application.runReadAction
import com.intellij.openapi.project.Project
import com.intellij.openapi.roots.impl.FilePropertyPusher
import com.intellij.openapi.util.io.FileTooBigException
import com.intellij.openapi.vfs.VirtualFile
import com.intellij.util.Base64
import com.intellij.util.indexing.FileBasedIndex
import com.intellij.util.indexing.FileContentImpl
import com.intellij.util.indexing.IndexedHashesSupport
import com.intellij.util.indexing.SubstitutedFileType

object IndexedFilePaths {

  const val TOO_LARGE_FILE = "<TOO LARGE>"

  const val FAILED_TO_LOAD = "<FAILED TO LOAD: %s>"

  fun createIndexedFilePath(fileOrDir: VirtualFile, project: Project): IndexedFilePath {
    val fileId = FileBasedIndex.getFileId(fileOrDir)
    val fileUrl = fileOrDir.url
    val fileType = if (fileOrDir.isDirectory) null else fileOrDir.fileType.name
    val substitutedFileType = if (fileOrDir.isDirectory) {
      null
    }
    else {
      runReadAction {
        SubstitutedFileType.substituteFileType(fileOrDir, fileOrDir.fileType, project).name.takeIf { it != fileType }
      }
    }
    val (contentHash, indexedHash) = try {
      val fileContent = FileContentImpl.createByFile(fileOrDir) as FileContentImpl
      runReadAction {
        val contentHash = IndexedHashesSupport.getBinaryContentHash(fileContent.content)
        val indexedHash = IndexedHashesSupport.calculateIndexedHash(fileContent, contentHash)
        Base64.encode(contentHash) to Base64.encode(indexedHash)
      }
    }
    catch (e: FileTooBigException) {
      TOO_LARGE_FILE to TOO_LARGE_FILE
    }
    catch (e: Throwable) {
      val msg = FAILED_TO_LOAD.format(e.message)
      msg to msg
    }

    val fileSize = if (fileOrDir.isDirectory) null else fileOrDir.length
    val portableFilePath = PortableFilePaths.getPortableFilePath(fileOrDir, project)
    val allPusherValues = dumpFilePropertyPusherValues(fileOrDir, project).mapValues { it.value?.toString() ?: "<null-value>" }
    return IndexedFilePath(
      fileId,
      fileType,
      substitutedFileType,
      fileSize,
      fileUrl,
      portableFilePath,
      allPusherValues,
      indexedHash,
      contentHash
    )
  }

  private fun dumpFilePropertyPusherValues(file: VirtualFile, project: Project): Map<String, Any?> {
    val map = hashMapOf<String, Any?>()
    FilePropertyPusher.EP_NAME.forEachExtensionSafe { pusher ->
      if (file.isDirectory && pusher.acceptsDirectory(file, project)
          || !file.isDirectory && pusher.acceptsFile(file, project)
      ) {
        map[pusher.pusherName] = pusher.getImmediateValue(project, file)
      }
    }
    return map
  }

  private val FilePropertyPusher<*>.pusherName: String
    get() = javaClass.name
      .removePrefix("com.")
      .removePrefix("intellij.")
      .removePrefix("jetbrains.")
      .replace("util.", "")
      .replace("impl.", "")
}
